;+
;
; Reading Data
;
; @param CMD {in}{required}{type=structure}
; only cmd.plt is used

; @param HOTYP {in}{required}{type=string}
;
; @param PLTTYP {in}{required}{type=string}
;
; @param DIMPLOT {in}{required}{type=integer}
;
; @param IOVER {in}{required}
;
; @keyword ALL_DATA
;
; @keyword USE_LARGE_DOMAIN
;
; @keyword _EXTRA
;
; @returns
; -1 in case of error
;
; @examples
;
; IDL> cmd={exp:'HH',timave:'1m',date1:'199201',plt:'xy',spec:'-',grid:'umath',var:'temp_1'}
; IDL> hotyp='-'
; IDL> plttyp='pltt'
; IDL> dimplot=1
; IDL> data_bases=['local:./'] ++
; IDL> result=data_read(cmd, hotyp, plttyp, dimplot, iover )
; IDL> print, result

; @uses
; <pro>common</pro>
; <propost_it>com_eg</propost_it>
; <propost_it>zoom_indices_debug</propost_it>
;
; <propost_it>def_dbase</propost_it>
;
; @todo
;
; use of iover ?
; init of data_bases in example
;
; @history
;
; - fplod 20091209T094824Z aedon.locean-ipsl.upmc.fr (Darwin)
;
;   * check parameters
;
; @version
; $Id$
;
;-
FUNCTION data_read, cmd, hotyp, plttyp, dimplot, iover $
         , ALL_DATA=all_data $
         , USE_LARGE_DOMAIN=use_large_domain $
         , _EXTRA=extra
;
  compile_opt idl2, strictarrsubs
;
@com_eg
;
; Return to caller if errors
 ON_ERROR, 2
;
   IF debug_w THEN BEGIN
    info = report('enter ...')
    print, '       cmd at top of data_read = ', cmd
   ENDIF

 usage='result=data_read( cmd, hotyp, plttyp, dimplot, iover ' $
         + ', ALL_DATA=all_data' $
         + ', USE_LARGE_DOMAIN=use_large_domain' $
         + ', _EXTRA=extra)'

;
 nparam = N_PARAMS()
 IF (nparam LT 4) THEN BEGIN
    ras = report(['Incorrect number of arguments.' $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
 ENDIF

 arg_type = size(cmd,/type)
 IF (arg_type NE 8) THEN BEGIN
   ras = report(['Incorrect arg type cmd' $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
 ENDIF

 arg_struct_tags=TAG_NAMES(cmd)

 tag=WHERE(STRMATCH(arg_struct_tags, 'TIMAVE'))
 IF (tag EQ -1) THEN BEGIN
   ras = report(['Incorrect arg tag TIMAVE cmd' $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
 ENDIF
 arg_type = size(cmd.timave,/type)
 IF (arg_type NE 7) THEN BEGIN
   ras = report(['Incorrect arg type cmd.timave' $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
 ENDIF
 tag=WHERE(STRMATCH(arg_struct_tags, 'DATE1'))
 IF (tag EQ -1) THEN BEGIN
   ras = report(['Incorrect arg tag DATE1 cmd' $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
 ENDIF
 arg_type = size(cmd.date1,/type)
 IF (arg_type NE 7) THEN BEGIN
   ras = report(['Incorrect arg type cmd.date1' $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
 ENDIF
 tag=WHERE(STRMATCH(arg_struct_tags, 'PLT'))
 IF (tag EQ -1) THEN BEGIN
   ras = report(['Incorrect arg tag PLT cmd' $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
 ENDIF

 arg_type = size(cmd.plt,/type)
 IF (arg_type NE 7) THEN BEGIN
   ras = report(['Incorrect arg type cmd.plt' $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
 ENDIF

 tag=WHERE(STRMATCH(arg_struct_tags, 'SPEC'))
 IF (tag EQ -1) THEN BEGIN
   ras = report(['Incorrect arg tag SPEC cmd' $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
 ENDIF
 arg_type = size(cmd.spec,/type)
 IF (arg_type NE 7) THEN BEGIN
   ras = report(['Incorrect arg type cmd.spec' $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
 ENDIF
 tag=WHERE(STRMATCH(arg_struct_tags, 'GRID'))
 IF (tag EQ -1) THEN BEGIN
   ras = report(['Incorrect arg tag GRID cmd' $
          + '!C' $
          + 'Usage : ' + usage])
   return, -1
 ENDIF
 arg_type = size(cmd.grid,/type)
 IF (arg_type NE 7) THEN BEGIN
   ras = report(['Incorrect arg type cmd.grid' $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
 ENDIF
 tag=WHERE(STRMATCH(arg_struct_tags, 'VAR'))
 IF (tag EQ -1) THEN BEGIN
   ras = report(['Incorrect arg tag VAR cmd' $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
 ENDIF
 arg_type = size(cmd.grid,/type)
 IF (arg_type NE 7) THEN BEGIN
   ras = report(['Incorrect arg type cmd.var' $
          + '!C' $
          + 'Usage : ' + usage])
   return, -1
 ENDIF
 tag=WHERE(STRMATCH(arg_struct_tags, 'EXP'))
 IF (tag EQ -1) THEN BEGIN
   ras = report(['Incorrect arg tag EXP cmd' $
          + '!C' $
          + 'Usage : ' + usage])
    return, -1
 ENDIF
 arg_type = size(cmd.exp,/type)
 IF (arg_type NE 7) THEN BEGIN
   ras = report(['Incorrect arg type cmd.exp' $
          + '!C' $
          + 'Usage : ' + usage])
   return, -1
 ENDIF

 arg_type = size(hotyp,/type)
 IF (arg_type NE 7) THEN BEGIN
   ras = report(['Incorrect arg type hotyp' $
          + '!C' $
          + 'Usage : ' + usage])
   return, -1
 ENDIF

 arg_type = size(plttyp,/type)
 IF (arg_type NE 7) THEN BEGIN
   ras = report(['Incorrect arg type plttyp' $
          + '!C' $
          + 'Usage : ' + usage])
   return, -1
 ENDIF

 arg_type = size(dimplot,/type)
 IF ((arg_type NE 2) AND (arg_type NE 3)) THEN BEGIN
   ras = report(['Incorrect arg type dimplot' $
          + '!C' $
          + 'Usage : ' + usage])
   return, -1
 ENDIF

 common_type=size(data_bases,/type)
 IF (common_type NE 7) THEN BEGIN
   ras = report(['Incorrect common type data_bases' $
          + '!C' $
          + 'Usage : ' + usage])
  return, -1
 ENDIF

 common_type=size(box_h,/type)
 IF ((common_type NE 2) AND (common_type NE 3)) THEN BEGIN
   ras = report(['Incorrect common type box_h' $
          + '!C' $
          + 'Usage : ' + usage])
   return, -1
 ENDIF
 common_nelem=size(box_h,/n_elements)
 IF (common_nelem NE 4) THEN BEGIN
   ras = report(['Incorrect common n_elements box_h' $
          + '!C' $
          + 'Usage : ' + usage])
   return, -1
 ENDIF

;
; same data as previous read ?


   read = 1

   IF cmd.var EQ cmd_prev.var AND cmd.grid EQ cmd_prev.grid AND cmd.plt EQ cmd_prev.plt AND cmd.timave EQ cmd_prev.timave AND cmd.date1 EQ cmd_prev.date1 THEN BEGIN 
      IF hotyp NE '-' THEN BEGIN 
                                ; time serie
         IF cmd.spec EQ cmd_prev.spec THEN BEGIN 
            fldr = field 
            read = 0
         ENDIF ELSE read = 1
         
      ENDIF ELSE BEGIN 
         read = 1
         IF strpos(cmd.plt, '#') NE -1 THEN BEGIN
            IF  cmd.plt EQ cmd_prev.plt THEN BEGIN ; not same zoom
               fldr = field 
               read = 0
            ENDIF ELSE begin
               read = 1
            ENDELSE
         ENDIF 
      ENDELSE 
   ENDIF ELSE read = 1
   
   read = 1
   
IF read EQ 1 THEN BEGIN 

   cmd_prev = cmd

; difference from cmd.exp ? division from exp ?

   diff_from_exp = 0
   div_from_exp = 0

   IF strpos(cmd.exp, '-') NE -1 THEN BEGIN
      argvar = strsplit(cmd.exp,'-', /EXTRACT)
   ENDIF ELSE IF strpos(cmd.exp, '/') NE -1 THEN BEGIN
      argvar = strsplit(cmd.exp,'/', /EXTRACT)
   ENDIF ELSE argvar = cmd.exp
      
   IF n_elements(argvar) EQ 2 THEN BEGIN
      IF strpos(cmd.exp, '-') NE -1 THEN BEGIN
         diff_from_exp = 1
         exp_init = cmd.exp
         cmd.exp = argvar[0]
      ENDIF
      IF strpos(cmd.exp, '/') NE -1 THEN BEGIN
         div_from_exp = 1
         exp_init = cmd.exp
         cmd.exp = argvar[0]
      ENDIF      
   ENDIF 
;   varexp = cmd.exp   

; define data base
  
   ncdf_db = def_dbase(cmd.exp)

   IF debug_w THEN print, '       varexp defined in data_read = ', varexp

; define file name

   def_file_name, cmd, ncdf_db, file_name, delta_t1

; get file if remote

   get_file, file_name, ncdf_db

; read grid from file if require (@<grid> option) 

   IF read_grid_from_file EQ 1 THEN BEGIN 
      mesh_from_file, cmd.grid, file_name, ncdf_db, cmd.var, ALL_DATA = all_data
      key_shift_map = key_shift
      file_name2 = file_name
      ncdf_db2 = ncdf_db
   ENDIF 

; define horizontal domain and vertical domain if needed (used in
; read_ncdf and update_data)

   IF keyword_set(use_large_domain) THEN BEGIN
      box_plot = box_h
   ENDIF ELSE box_plot = def_box(cmd.plt, dimplot, legbox, time_stride)
   small_domain = cmd.plt
   CASE n_elements(box_plot) OF
      4 : IF vert_switch GE 1 THEN box_plot = [box_plot, vert_mean]
      6 : IF vert_switch GE 1 THEN box_plot = [box_plot[0:3], vert_mean] ELSE box_plot = box_plot[0:3]
   ENDCASE

; Exceptions : need to read all the vertical data (and not a subset)
; for the pltz case. Other cases ?   
   IF plttyp EQ 'pltz' THEN box_plot = box_plot[0:3]

; define timesteps time1 and time2

   IF hotyp NE '-' THEN BEGIN
      time1 = delta_t1+1
      timearr = compute_time(cmd.timave, cmd.date1, cmd.spec)
      time2 = timearr.count+delta_t1
      time = timearr.scale
   ENDIF ELSE BEGIN
      IF strpos(cmd.timave, 'mm') NE -1 THEN time1 = delta_t1+long(strmid(cmd.date1, 0, 2)) $
      ELSE  time1 = delta_t1+1
      time2 = time1
      time = time1
   ENDELSE

; save time in common fld_att

   time1_r = time1
   time2_r = time2

; call specific read routine

   CASE STRMID(cmd.var, 0, 2) OF
      '@@': BEGIN
         IF debug_w THEN print, 'keyword_set(all_data) : ', keyword_set(all_data)
         fldr = macro_read(file_name, cmd.var, ncdf_db, BOXZOOM = box_plot, TIME_1 = time1, TIME_2 = time2, ALL_DATA = all_data, _EXTRA=extra)
         IF stddev_diff EQ 1 THEN fldr.origin = 'diff'
      END 
      ELSE: BEGIN
         CASE plttyp OF
            'yfx': BEGIN        ; scatter plot y=f(x)
               ; read 2 fields
               idx = strpos(cmd.var, '=f(')
               @two_var_read
               ; custom stucture for scatter plot
               fldr = {name: var1, data: fldr1.data, legend: fldr1.legend, $
                       units:  fldr1.units, origin: fldr1.origin, dim: fldr1.dim, $
                       name2: var2, data2: fldr2.data, legend2: fldr2.legend, $
                       units2:  fldr2.units, origin2: fldr2.origin, dim2: fldr2.dim}
               ; diff not possible yet
               IF diff_from_exp EQ 1 OR STRMID(cmd.spec, 0, 2) EQ 'd:' THEN BEGIN
                  diff_from_exp = 0
                  cmd.spec = '-'
                  print, '   data_read : difference in scatter plot not ready'
               ENDIF 
               ; re-organise cmd.var
               cmd.var = var1
               cmd.var2 = var2
               
            END
            'ybinx': BEGIN
               ; check how many fields to read
               sl_pos = strpos(cmd.var, '/')
               var3_ybinx = ""
               IF sl_pos NE -1 THEN BEGIN 
                  eq_pos = strpos(cmd.var, '=')
                  str_len = strlen(cmd.var)
                  var1 = strmid(cmd.var, 0, sl_pos)
                  var3_ybinx = strmid(cmd.var, sl_pos+1, eq_pos-sl_pos-1)
                  cmd.var = var1+strmid(cmd.var, eq_pos, str_len)
               ENDIF 
               ; read 2 fields
               idx = strpos(cmd.var, '=bin(')
               @two_var_read
               
               ; if two grids different then stop for now (best to interpolate on same grid)

  ;             IF grid2_test_ybinx NE cmd.grid THEN BEGIN 
  ;                print, '  *** WARNING !!!! ybinx not ready for different grids'
  ;                print, '  --> use interpolation to have fields on same grid'
  ;                return, -1 
  ;             ENDIF 

               ; if third field needed, then read
               IF var3_ybinx NE "" THEN BEGIN 
                  IF strpos (cmd_wrk.grid, '#') NE -1 THEN BEGIN 
                     varpos = strpos(file_name, var1)
                     file_name3= strmid(file_name, 0, varpos)+var3_ybinx+'.nc'
                     IF debug_w THEN print, '   file_name3 = ', file_name3
                  ENDIF ELSE file_name3 = file_name
                  box_plot = def_box(small_domain, dimplot, legbox, time_stride)
                  fldr3 = nc_read(file_name3, var3_ybinx, ncdf_db, BOXZOOM = box_plot, $
                                  TIME_1 = time1, TIME_2 = time2, ALL_DATA = all_data, _EXTRA=extra)
                  datyp3 = datyp
                  vargrid3 = vargrid1
                  ; custom stucture for bin plot
                  fldr = {name: var1, data: fldr1.data, legend: fldr1.legend, $
                          units:  fldr1.units, origin: fldr1.origin, dim: fldr1.dim, $
                          name2: var2, data2: fldr2.data, legend2: fldr2.legend, $
                          units2:  fldr2.units, origin2: fldr2.origin, dim2: fldr2.dim, $
                          name3: var3_ybinx, data3: fldr3.data, legend3: fldr3.legend, $
                          units3:  fldr3.units, origin3: fldr3.origin, dim3: fldr3.dim}
               ENDIF ELSE BEGIN
                  ; custom stucture for bin plot
                  fldr = {name: var1, data: fldr1.data, legend: fldr1.legend, $
                          units:  fldr1.units, origin: fldr1.origin, dim: fldr1.dim, $
                          name2: var2, data2: fldr2.data, legend2: fldr2.legend, $
                          units2:  fldr2.units, origin2: fldr2.origin, dim2: fldr2.dim}
               ENDELSE  
               
               ; re-organise cmd.var
               cmd.var = var1
               cmd.var2 = var2
            END 
            ELSE: BEGIN
               IF vecplot EQ 1 THEN BEGIN ; vectors case
                  idx = strpos(cmd.var, ',')
                  var1 = strmid(cmd.var, 1, idx-1)
                  var2 = strmid(cmd.var, idx+1, strlen(cmd.var)-idx-2)
;                  print, var1, var2
;old version                  file_nam = strmid(file_name, 0, strlen(file_name)-4)
                  file_nam = strmid(base_file_name+base_suffix, 0, strlen(base_file_name+base_suffix)-1)
                  vargrid = strmid(cmd.grid, 0, 1)
                  CASE STRMID(var1, 0, 2) OF
                     '@@': fldr1 = macro_read(file_name, var1, ncdf_db, BOXZOOM = box_plot, $
                                              TIME_1 = time1, TIME_2 = time2, _EXTRA=extra)
                     ELSE: fldr1 = nc_read(file_nam+strmid(cmd.grid, 0, 1)+suff_domain+'.nc', var1, ncdf_db, $
                                           BOXZOOM = boxplot, TIME_1 = time1, TIME_2 = time2, $
                                           ALL_DATA = all_data, _EXTRA=extra)
                  ENDCASE 
                  vargrid = strmid(cmd.grid, 1, 1)
                  CASE STRMID(var2, 0, 2) OF
                     '@@': fldr2 = macro_read(file_name, var2, ncdf_db, BOXZOOM = box_plot, $
                                              TIME_1 = time1, TIME_2 = time2, _EXTRA=extra)
                     ELSE: fldr2 = nc_read(file_nam+strmid(cmd.grid, 1, 1)+suff_domain+'.nc', var2, ncdf_db, $
                                           BOXZOOM = box_plot, TIME_1 = time1, TIME_2 = time2, $
                                           ALL_DATA = all_data, _EXTRA=extra)
                  ENDCASE     
                                ; consistency check
                  IF fldr1.dim NE fldr2.dim THEN BEGIN
                     print, '  *** dimension inconsistency in vector plot : ', cmd.var
                     return, -1
                  ENDIF 
                ; custom stucture for vector plot
                  fldr = {name: var1, data: fldr1.data, legend: fldr1.legend, $
                          units:  fldr1.units, origin: fldr1.origin, dim: fldr1.dim, $
                          name2: var2, data2: fldr2.data, legend2: fldr2.legend, $
                          units2:  fldr2.units, origin2: fldr2.origin, dim2: fldr2.dim}
                  IF diff_from_exp EQ 1 OR STRMID(cmd.spec, 0, 2) EQ 'd:' THEN BEGIN
                     diff_from_exp = 0
                     cmd.spec = '-'
                     print, '   data_read : difference in vector plot not ready'
                  ENDIF 
                                ; re-organise cmd.var
                  cmd.var = var1
                  cmd.var2 = var2
              

               ENDIF ELSE BEGIN  ; general case
                  fldr = nc_read(file_name, cmd.var, ncdf_db, BOXZOOM = box_plot, $
                                  TIME_1 = time1, TIME_2 = time2, ALL_DATA = all_data, _EXTRA=extra)
                  ;fldr = nc_read(file_name, cmd.var, ncdf_db, $
                  ;               TIME_1 = time1, TIME_2 = time2, ALL_DATA = all_data, _EXTRA=extra)              
                  ; perform mean if hovmoeller diagram is required
                  ; It is absolutely necessary for the division case with 
                  ; a hovmoeller diagram (before performing the division)
                  IF plttyp EQ 'pltt' AND div_from_exp EQ 1 THEN BEGIN
                     mask_z, fldr.data, cmd, boite_pltt, dimplot, legz
                     z2d = checkfield(fldr.data, plttyp, TYPE = hotyp, BOXZOOM = boite_pltt, DIREC = direc, _EXTRA=extra)
                     fldr = { name : fldr.name, data : z2d, legend : fldr.legend, units : fldr.units, $
                               origin : fldr.origin, dim : size(z2d, /N_DIMENSIONS), direc : hotyp}
                     print, 'Averaging made before the call to SAXO plt routines !!!!'
                  ENDIF
               ENDELSE 
            END 
         ENDCASE  
      END  
   ENDCASE  

; Redefinition of time because it is updated in read_ncdf
; Useful for pltt routine

   IF hotyp NE '-' THEN BEGIN
      time = timearr.scale
   ENDIF ELSE BEGIN
      time = time1
   ENDELSE


; density projection

   IF splot EQ 1 THEN BEGIN
      IF cmd.timave EQ '1y' AND 1 THEN BEGIN 
         print, 'data_read:  Constructing annual mean from monthly means...'
         cmd.timave = '1m'
         year_s = cmd.date1
         cumulative = 0.
         FOR i = 1, 12 DO BEGIN
            IF i EQ 1  THEN cmd.date1 = year_s + '01'
            IF i EQ 2  THEN cmd.date1 = year_s + '02'
            IF i EQ 3  THEN cmd.date1 = year_s + '03'
            IF i EQ 4  THEN cmd.date1 = year_s + '04'
            IF i EQ 5  THEN cmd.date1 = year_s + '05'
            IF i EQ 6  THEN cmd.date1 = year_s + '06'
            IF i EQ 7  THEN cmd.date1 = year_s + '07'
            IF i EQ 8  THEN cmd.date1 = year_s + '08'
            IF i EQ 9  THEN cmd.date1 = year_s + '09'
            IF i EQ 10 THEN cmd.date1 = year_s + '10'
            IF i EQ 11 THEN cmd.date1 = year_s + '11'
            IF i EQ 12 THEN cmd.date1 = year_s + '12'
            print, '   Date1 = ', cmd.date1
            sfild = fldr
            bin_sigma, cmd, sfild, BOXZOOM = box_plot, ALL_DATA = all_data ;;;;;;;;;;;;;; prob
            cumulative = cumulative + sfild.data
         ENDFOR
         fldr = sfild
         fldr.data = cumulative/12.
         cmd.timave = '1y'
         cmd.date1 = year_s
      ENDIF ELSE BEGIN
         IF cmd.timave EQ '1m' AND strmid(cmd.plt, 0, 2) EQ 'st' AND really_1m_st EQ 1 THEN BEGIN
            sfild = fldr
            timedim = (size(sfild.data))[(size(sfild.data))[0]]
            fldrd = fltarr((size(sfild.data))[1],(size(sfild.data))[2], (sig_max - sig_min)/sig_del + 1, timedim)
            FOR i = 0, timedim-1 DO BEGIN
               print, 'data_read: Performing monthly bining. Indices: first, last, current = 0,', timedim-1, i
               sfild2 = {name: sfild.name, data: sfild.data[*, *, *, i], legend: sfild.legend, units: sfild.units, origin: sfild.origin, dim: sfild.dim-1}
               bin_sigma, cmd, sfild2, BOXZOOM = box_plot, ALL_DATA = all_data 
               fldrd[*, *, *, i]= sfild2.data
            ENDFOR 
            fldr = {name: sfild.name, data:fldrd, legend: sfild.legend, units: sfild.units, origin: sfild.origin, dim: sfild.dim}
         ENDIF ELSE BEGIN
            sfild = fldr
            bin_sigma, cmd, sfild, BOXZOOM = box_plot, ALL_DATA = all_data
            ;;bin_sigma, cmd, sfild, ALL_DATA = all_data
            fldr = sfild
         ENDELSE 
      ENDELSE 
   ENDIF 

; modify data info if needed (actual data modification done in; trends.pro called by pltt.pro)

   CASE plttyp OF
      'pltt': BEGIN
         IF run_stddev EQ 0 THEN BEGIN
            CASE strmid(cmd.trend, 0, 1) OF
               '1': fldr.origin = 'diff'
               '2': fldr.origin = 'diff'
               '3': fldr.origin = 'diff'
               '4': fldr.origin = 'diff'
               ELSE:
            ENDCASE
         ENDIF
      END 
      ELSE:
   ENDCASE 
   
   IF n_elements(fldr.data) EQ 1 THEN return, fldr

; Difference 2 cases : from cmd.exp or from cmd.spec


   IF diff_from_exp EQ 1 THEN BEGIN 

      cmd.exp = argvar[1]
      print, '    remove : ', cmd.exp

      ; read field2
      field2 = data_read(cmd, hotyp, plttyp, dimplot, iover, ALL_DATA = all_data, _EXTRA=extra)
      IF n_elements(field2.data) EQ 1 THEN return, field2

      ; perform difference
      diff = fldr.data-field2.data
      IF (where(fldr.data EQ valmask))[0] NE -1 THEN $
       diff[where(fldr.data EQ valmask)] = valmask
      fldr.data = diff
      fldr.origin = 'diff'   
      cmd.exp = exp_init
      varexp = cmd.exp 

   ENDIF ELSE IF div_from_exp EQ 1 THEN BEGIN  ; Division : 1 case
      
      cmd.exp = argvar[1]
      print, '    divide by : ', cmd.exp

      div = fldr.data
      
      ; read field2
      field2 = data_read(cmd, hotyp, plttyp, dimplot, iover, ALL_DATA = all_data, _EXTRA=extra)
      IF n_elements(field2.data) EQ 1 THEN return, field2

      IF plttyp EQ 'pltt' THEN BEGIN
         mask_z, field2.data, cmd, boite_pltt, dimplot, legz
         z2d = checkfield(field2.data, plttyp, TYPE = hotyp, BOXZOOM = boite_pltt, DIREC = direc, _EXTRA=extra)
         field2 = { name : field2.name, data : z2d, legend : field2.legend, units : field2.units, $
                   origin : field2.origin, dim : size(z2d, /N_DIMENSIONS), direc : hotyp}
         print, 'Averaging made before the call to SAXO plt routines !!!!'
      ENDIF
      
      ; perform division
      IF (where(field2.data EQ 0.0))[0] NE -1 THEN BEGIN
         print, 'Be careful some data are null. They will be masked ! '
         idx0 = where(field2.data EQ 0.0)
         field2.data[idx0] = valmask
         fldr.data[idx0] = valmask
         div[idx0] = valmask
      ENDIF

      idx = where(field2.data NE valmask)
      div[idx] = fldr.data[idx] / field2.data[idx]

      fldr.data = div
      fldr.origin = 'div'   
      cmd.exp = exp_init
      varexp = cmd.exp 
      
   ENDIF ELSE BEGIN

; If difference, decode second field, read and perform difference

      IF STRMID(cmd.spec, 0, 2) EQ 'd:' THEN BEGIN
         
         cmd2 = cmd
         cmdl = STRMID(cmd.spec, 2, strlen(cmd.spec)-2)
         
         print, '    remove : ', cmdl
         
         ; decode field2
         argvar = strsplit(cmdl, '/', /EXTRACT)

         cmd2.exp = argvar[0]
         cmd2.timave = argvar[1]
         cmd2.date1 = argvar[2]
         cmd2.spec = '-'

         IF n_elements(argvar) EQ 4 THEN cmd2.spec = argvar[3] 
         
         ; read field2
         field2 = data_read(cmd2, hotyp, plttyp, dimplot, iover, ALL_DATA = all_data, _EXTRA=extra)
         IF n_elements(field2.data) EQ 1 THEN return, field2

         ; perform difference
         diff = fldr.data-field2.data
         IF (where(fldr.data EQ valmask))[0] NE -1 THEN $
          diff[where(fldr.data EQ valmask)] = valmask
         fldr.data = diff
         fldr.origin = 'diff'   
         cmd.exp = cmd.exp+' - '+cmd2.exp+ ' ('+cmd2.timave+' '+cmd2.date1+')'
         varexp = cmd.exp 

      ENDIF 

   ENDELSE

ENDIF ELSE BEGIN

   print, ''
   print, '   Data already read (previous field)'
   print, ''
   
ENDELSE

@zoom_indices_debug

; Pb with varexp which is always updated in read_ncdf
; For Seb, varexp is the name of the file. For Eric, varexp is the name of the experiment
varexp = cmd.exp

   IF debug_w THEN BEGIN
    print, '       varexp= ', varexp
    info = report('leaving ...')
   ENDIF


return, fldr

END 
